﻿using System.Windows;
using System.ComponentModel.Composition;
using System.ComponentModel.Composition.Registration;
using System.ComponentModel.Composition.Hosting;
using System.Reflection;
using VSMMefWpfDemo.Business.Validator;
using VSMMefWpfDemo.Business.Entities;


namespace VSMMefWpfDemo
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private CompositionContainer _container;

        public IValidateEntity<Person> PersonValidator  { get; set; }

        private readonly Person _model = new Person();

        public MainWindow()
        {
            InitializeComponent();
            DataContext = _model;
            RegisterMefParts();
        }

        private void RegisterMefParts()
        {
            var conventions = new RegistrationBuilder();
            conventions.ForTypesDerivedFrom<IValidateEntity<Person>>()
                       .Export<IValidateEntity<Person>>();

            // conventions.ForTypesMatching(x => x.Namespace.Contains("Business.Validator")).ExportInterfaces(x => x.IsPublic);

            conventions.ForType<MainWindow>().ImportProperty(x => x.PersonValidator);


            Assembly businessAssembly = typeof (IValidateEntity<>).Assembly;
            var catalog = new AggregateCatalog();
            catalog.Catalogs.Add(new AssemblyCatalog(Assembly.GetExecutingAssembly()));
            catalog.Catalogs.Add(new AssemblyCatalog(businessAssembly, conventions));

            _container = new CompositionContainer(catalog,
                                                  CompositionOptions.DisableSilentRejection |
                                                  CompositionOptions.IsThreadSafe);

            try
            {
                _container.SatisfyImportsOnce(this, conventions);
            }
            catch (CompositionException ex)
            {
                MessageBox.Show((ex.Message));
            }
        }

        private void ValidateButton_Click(object sender, RoutedEventArgs e)
        {
            Status.Text = PersonValidator.Validate(_model) ? "Person is valid." : "Person isn't valid.";
        }
    }
}
